from model.editable import Editable
from model.translation import from_container_list


class Example(Editable):
    def __init__(self, example_xml, cluster_info):
        self.translations = from_container_list(example_xml.querySelectorAll("translationContainer"))
        
        inner_xml = example_xml.querySelector("corpusExample")
        if inner_xml is not None:
            self.inner = CorpusExample(inner_xml)
        else:
            inner_xml = example_xml.querySelector("multiwordExample") 
            self.inner = MultiwordExample(inner_xml, cluster_info)
            
        all_components = [ComponentLexeme(el) for el in inner_xml.childNodes]
        self.components = [comp for comp in all_components if comp.isValid()]
    
        
    def export(self, doc):
        result = doc.createElement("exampleContainer")
        
        inner = self.inner.export(doc)
        # TODO: bad quick fix
        for comp in self.components:
            inner.appendChild(comp.export(doc))
        
        result.appendChild(inner)
        return result
    
    def text(self):
        return " ".join([comp.text for comp in self.components])
        
    def get_cluster(self):
        return self.inner.get_cluster()
    
    def set_cluster(self, cluster):
        self.inner.cluster = cluster
    
    def is_collocation(self):
        return self.get_view_type() == 2
    
    def get_view_type(self):
        # as per the bosses, these are the rules for different colors
        if type(self.inner) is CorpusExample:
            return 1
        elif self.inner.type == "collocation":
            return 2
        else:
            return 3
            
            
class CorpusExample:
    def __init__(self, example_xml):
        self.other_attributes = {}
        for oth_attr in ["example_id", "modified", "lexical_unit_id", "audio"]:
            if example_xml.hasAttribute(oth_attr):
                self.other_attributes[oth_attr] = example_xml.getAttribute(oth_attr)
            
    def export(self, doc):
        result = doc.createElement("corpusExample")
        for key, value in self.other_attributes.items():
            result.setAttribute(key, value)
        return result
    
    def get_cluster(self):
        return None
    

class MultiwordExample:
    def __init__(self, example_xml, cluster_info):
        self.other_attributes = {}
        for oth_attr in ["lexical_unit_id", "structure_id", "structureName", "audio", "frequency", "logDice"]:
            if example_xml.hasAttribute(oth_attr):
                self.other_attributes[oth_attr] = example_xml.getAttribute(oth_attr)
        
        self.cluster = self._determine_cluster_number(example_xml, cluster_info)
        
        if example_xml.hasAttribute("type"):
            self.type = example_xml.getAttribute("type")
        else:
            self.type = None
        
    def _determine_cluster_number(self, example_xml, cluster_info):
        # since cluster numbers can be fairly fragmented, this is defragmentation
        # we do need to get info about sense examples to do this, that is why we have cluster_info
        if cluster_info is None:
            cluster_mappings, cluster_begin = {}, 0
        else:
            cluster_mappings, cluster_begin = cluster_info
        
            
        if not example_xml.hasAttribute("cluster"):
            cluster = len(cluster_mappings) + cluster_begin
            cluster_mappings[cluster] = cluster
        else:
            cluster = int(example_xml.getAttribute("cluster"))
            if cluster not in cluster_mappings:
                cluster_mappings[cluster] = len(cluster_mappings) + cluster_begin
         
        return cluster_mappings[cluster]
            
    def export(self, doc):
        result = doc.createElement("multiwordExample")
            
        for key, value in self.other_attributes.items():
            result.setAttribute(key, value)
        
        if self.cluster_valid:
            result.setAttribute("cluster", str(self.cluster))
        
        if self.type is not None:
            result.setAttribute("type", self.type)
        
        return result
        
    def get_cluster(self):
        return self.cluster


class ComponentLexeme(Editable):
    def __init__(self, xml):
        self.other_attributes = {}
        
        if xml.nodeName == "#text":
            self.text = xml.data
            self.role = None
        else:
            self.text = xml.textContent
            self.role = xml.getAttribute("role")

            for oth_attr in ["lexical_unit_lexeme_id", "slolex", "kol"]:
                if xml.hasAttribute(oth_attr):
                    self.other_attributes[oth_attr] = xml.getAttribute(oth_attr)
        
        self.text = self.text.strip()
    
    def isValid(self):
        return len(self.text) > 0
    
    def export(self, doc):
        if self.role is None:
            return doc.createTextNode(self.text)
        
        result = doc.createElement("comp")
        result.setAttribute("role", self.role)
        result.textContent = self.text
        
        for key, value in self.other_attributes.items():
            result.setAttribute(key, value)
        
        return result
    

